#ifndef BatchLab_Main_h
#define BatchLab_Main_h

// Inheritance :
#include <AIDA/IAnalysisFactory.h>

#include <AIDA/IConstants.h> //For AIDA_VERSION

#include <vector>

namespace Slash {namespace Core {class IWriter;}}
namespace Slash {namespace Core {class ISession;}}

namespace BatchLab {

// The Main handles a concrete Slash::Core::ISession and 
// is also a proxy to a BatchLab::AnalysisFactory.

class Main : public virtual AIDA::IAnalysisFactory {
public: //AIDA/IAnalysisFactory
#if (AIDA_VERSION_MAJOR<=3) && (AIDA_VERSION_MINOR<=2) 
  virtual AIDA::ITreeFactory* createTreeFactory();
  virtual AIDA::IHistogramFactory* createHistogramFactory(AIDA::ITree&);
  virtual AIDA::IDataPointSetFactory* createDataPointSetFactory(AIDA::ITree&);
  virtual AIDA::ITupleFactory* createTupleFactory(AIDA::ITree&);
  virtual AIDA::IFunctionFactory* createFunctionFactory(AIDA::ITree&);
  virtual AIDA::IFitFactory* createFitFactory();
  virtual AIDA::IPlotterFactory* createPlotterFactory
    (int = 0,char** = 0,const std::string& = "",const std::string& = "");
#else
  virtual AIDA::ITreeFactory* createTreeFactory(const std::string& = "");
  virtual AIDA::IHistogramFactory* createHistogramFactory(AIDA::ITree&,const std::string& = "");
  virtual AIDA::IDataPointSetFactory* createDataPointSetFactory(AIDA::ITree&,const std::string& = "");
  virtual AIDA::ITupleFactory* createTupleFactory(AIDA::ITree&,const std::string& = "");
  virtual AIDA::IFunctionFactory* createFunctionFactory(AIDA::ITree&,const std::string& = "");
  virtual AIDA::IFitFactory* createFitFactory(const std::string& = "");
  virtual AIDA::IPlotterFactory* createPlotterFactory
    (int = 0,char** = 0,const std::string& = "",const std::string& = "",const std::string& = "");

  virtual AIDA::IGenericFactory* createGenericFactory(const std::string& factoryType, const std::string& options = "");
  virtual AIDA::IGenericFactory* createManagedObjectGenericFactory(const std::string& factoryType,AIDA::ITree& tree,const std::string& options = "");

#endif
public:
  //Main();
  Main(const std::vector<std::string>& args = std::vector<std::string>());
  virtual ~Main();
  bool isValid() const;
private:
  Slash::Core::IWriter* fPrinter;
  Slash::Core::ISession* fSession;
  AIDA::IAnalysisFactory* fAnalysisFactory;
  int fVerboseLevel;
  bool fBalance;
};

}


#endif
